% Simulation of 
% Bernanke Gertler Gilchrist "The Financial Accelerator in a Quantitative Business-Cycle Framework."
% originally written by
% Roberto Croce, Ohio State University, Sept 2008; Modified by Stefano Neri on September 4, 2009
% this modification by L. Rojas; november 2013.
% bgg.mod, version 2.1. Code extended with a simulation with all the shocks

% Compares the impulse responses to government spending, technology and monetary policy shocks in the models 
% with and without the financial accelerator

param_ff = [ 0, 0.04 ] ; % Set the parameter measuring the sensitivity of the external finance premium to entrepreneurs' leverage
                         % zero corresponds to the model without the financial accelerator
                         
shock = input('Which shock? Government spending = 1; Technology = 2; Monetary policy = 3    '); % Select the type of shock


for ii = 1:2; % loop over the two values for the parameter measuring the sensitivity of the external finance premium to 
              % entrepreneurs' leverage
    
    var y c i g ce n rk r q k x a h pi rn ep;    % Declare all the variables in the model, including the shocks processes
    
    varexo e_rn e_g e_a;                      % Declare the exogenous variables, i.e. the innovations to the shocks g and a and the monetary shock
    
    % declare the parameters of the model
    
    parameters beta alpha delt omeg GY CY IY CEY Y_N X rho_a sig_a rho_g sig_g psi K_N gam mu nu theta rho sig;
    
    % Set the parameters
    
    CY      =	0.605770191     ; %  households' consumption - output ratio in steady state
    CEY  	=	0.01	        ; %  entrepreneurs' consumption - output ratio in steady state
    IY  	=	0.184229809     ; %  investment - output ratio in steady state
    GY  	=	0.2             ; %  government spending - output ratio in steady state
    K_N 	=	2.081759973     ; %  capital to net worth ratio in steady state
    Y_N 	=	0.282494996     ; %  output to net worth ratio in steady state
    X   	=	1.1             ; %  steady state firms' markup over marginal costs
    beta    =   0.99            ; %  households' discount factor
//  R       =   1/beta          ; %  Steady state nominal interest rate
    alpha   =   0.2             ; %  Capital share in the production function
//  eta     =   1/alpha         ;
    omeg    =   0.99            ; % Share of households' labor in production function. 1-omeg is the share of entrepreneurs'labor
    delt    =   0.025           ; % Depreciation rate of capital stock
    rho_a   =   0.99            ; % Persistene of technology process
    rho_g   =   0.95            ; % Persistene of government spending process
    psi     =   0.25            ; % Parameter in the adjustment cost function on capital
//  Rk      =   R + 0.02        ; % Return on capital = risk-free rate R + spread of 2 per cent per annum
    gam     =   1 - 0.0272      ; % Survival probability of entrepreneurs
    mu      =   0.12            ; % Monitoring cost
    theta   =   0.75            ; % Calvo probability of not being able to adjust prices
    rho     =   0.96            ; % Interest rate intertia: coefficient on rn(t-1)
    sig     =   0.20            ; % Response of monetary policy to inflation (t-1)
    nu      =   param_ff(ii)    ; % sensitivity of the external finance premium to ratio of net worth to value of capital
    
//  kap     =  ((1-theta)/theta)*(1-theta*beta);
//  eps     =   (1-delt)/((1-delt) + ((alpha/X)*(Y_N/K_N)));
    
    
    model(linear); % model specification; the model is already in linear form

    %********L********L*********L*********L********
    % parameters or steady state values that are (known) functions of other parameters (needed for estimation)
    %********L********L*********L*********L********
 
# R=1/beta; 
# eta=1/alpha;
# Rk =   1/beta + 0.02        ; % Return on capital = risk-free rate R + spread of 2 per cent per annum
# kap     =  ((1-theta)/theta)*(1-theta*beta);
# eps     =   (1-delt)/((1-delt) + ((alpha/X)*(Y_N/K_N)));
    
    % **************************************************************************
    % equations numbers correspond to the BGG (1999) NBER Working Paper No. 6455
    % **************************************************************************
    
    y             =  CY*c + IY*i + GY*g + CEY*ce                         ; % (4.14) RESOURCE CONSTRAINT
    
    c             =  -r + c(+1)                                          ; % (4.15) EULER EQUATION
    
    ce            =  n                                                   ; % (4.16) ENTREPRENEURS' CONSUMPTION
    
    rk(+1) - r    =  -nu*(n -(q + k))                                    ; % (4.17) RELATION BETWEEN EXTERNAL FINANCE PREMIUM AND NETWORTH
    
    rk            =  (1-eps)*(y - k(-1) - x) + eps*q - q(-1)             ; % (4.18) RETURN ON CAPITAL
    
    q             =  psi*(i - k(-1))                                     ; % (4.19)  q(+1) = psi*(i(+1)-k) PRICE OF NEWLY INSTALLED CAPITAL
    
    y             =  a + alpha*k(-1) + (1-alpha)*omeg*h                  ; % (4.20)  PRODUCTION FUNCTION
    
    y - h - x - c =  alpha*h                                             ; % (4.21)  LABOUR MARKET EQUILIBRIUM
    
    pi            =  kap*(-x) + beta*pi(+1)                              ; % (4.22)  NEW KEYNESIAN PHILLIPS CURVE
    
    k             =  delt*i(-1) + (1-delt)*k(-1)                         ; % (4.23) CAPITAL ACCUMULATION LAW
    
    n             =  gam*R*K_N*(rk - r) + r + n(-1)                      ; % (4.24) ACCUMULATION OF NETWORTH
    
    rn            =  rho*rn(-1) + sig*pi(-1) + e_rn                      ; % (4.25)  MONETARY POLICY RULE
    
    g             =  rho_g*g(-1) + e_g                                   ; % (4.26)  GOVERNMENT SPENDING PROCESS
    
    a             =  rho_a*a(-1) + e_a                                   ; % (4.27)  TECHNOLOGY PROCESS
    
    rn            =  r + pi(+1)                                          ; % EX-ANTE REAL INTEREST REATE
    
    ep            = -nu*(n -(q + k))                                     ; % definition external finance premium
    
end;        % ends the declaration of the model




steady;     % Computes the steady state of the model; in our case the steady state must be zero for all the variables

check;      % Checks for uniqueness of a stable equilibrium in neighborhood of steady state


if ii == 1; % without the financial accelerator
    
    
    if shock == 1; % government spending shock
        
        shocks;
        var e_g; stderr 0.01;                
        end;
    
    options_.nograph   = 1;
    options_.nomoments = 1;
    options_.noprint   = 1;
    
    stoch_simul(irf=24) y c i n pi rn q ce rk r k ep;
    
    irf_y_noff  = oo_.irfs.y_e_g;
    irf_c_noff  = oo_.irfs.c_e_g;
    irf_i_noff  = oo_.irfs.i_e_g;
    irf_pi_noff = oo_.irfs.pi_e_g;
    irf_rn_noff = oo_.irfs.rn_e_g;
    irf_r_noff  = oo_.irfs.r_e_g;
    irf_rk_noff = oo_.irfs.rk_e_g;
    irf_n_noff  = oo_.irfs.n_e_g;
    irf_q_noff  = oo_.irfs.q_e_g;
    irf_ep_noff = oo_.irfs.ep_e_g;
    
    end;

if shock == 2;  % technology shock
    
    shocks;
    var e_a; stderr 0.01;              
end;

options_.nograph   = 1;
options_.nomoments = 1;
options_.noprint   = 1;

stoch_simul(irf=24) y c i n pi rn q ce rk r k ep;

irf_y_noff  = oo_.irfs.y_e_a;
irf_c_noff  = oo_.irfs.c_e_a;
irf_i_noff  = oo_.irfs.i_e_a;
irf_pi_noff = oo_.irfs.pi_e_a;
irf_rn_noff = oo_.irfs.rn_e_a;
irf_r_noff  = oo_.irfs.r_e_a;
irf_rk_noff = oo_.irfs.rk_e_a;
irf_n_noff  = oo_.irfs.n_e_a;
irf_q_noff  = oo_.irfs.q_e_a;
irf_ep_noff = oo_.irfs.ep_e_a;

end

if shock == 3;   % monetary policy shock
    
    shocks;
    var e_rn; stderr 0.0025;        
end;

options_.nograph   = 1;
options_.nomoments = 1;
options_.noprint   = 1;

stoch_simul(irf=24) y c i n pi rn q ce rk r k ep;

irf_y_noff  = oo_.irfs.y_e_rn;
irf_c_noff  = oo_.irfs.c_e_rn;
irf_i_noff  = oo_.irfs.i_e_rn;
irf_pi_noff = oo_.irfs.pi_e_rn;
irf_rn_noff = oo_.irfs.rn_e_rn;
irf_r_noff  = oo_.irfs.r_e_rn;
irf_rk_noff = oo_.irfs.rk_e_rn;
irf_n_noff  = oo_.irfs.n_e_rn;
irf_q_noff  = oo_.irfs.q_e_rn;
irf_ep_noff = oo_.irfs.ep_e_rn;

end

end


if ii == 2; % with the financial accelerator
    
    if shock == 1;  % government spending shock
        
        shocks;
        var e_g; stderr 0.01;               
    end;
    
    options_.nograph   = 1;
    options_.nomoments = 1;
    options_.noprint   = 1;
    
    stoch_simul(irf=24) y c i n pi rn q ce rk r k ep;
    
    irf_y  = oo_.irfs.y_e_g;
    irf_c  = oo_.irfs.c_e_g;
    irf_i  = oo_.irfs.i_e_g;
    irf_pi = oo_.irfs.pi_e_g;
    irf_rn = oo_.irfs.rn_e_g;
    irf_r  = oo_.irfs.r_e_g;
    irf_rk = oo_.irfs.rk_e_g;
    irf_n  = oo_.irfs.n_e_g;
    irf_q  = oo_.irfs.q_e_g;
    irf_ep = oo_.irfs.ep_e_g;
    
end;

if shock == 2;  % technology shock
    
    shocks;
    var e_a; stderr 0.01;             
end;

options_.nograph   = 1;
options_.nomoments = 1;
options_.noprint   = 1;

stoch_simul(irf=24) y c i n pi rn q ce rk r k ep;

irf_y  = oo_.irfs.y_e_a;
irf_c  = oo_.irfs.c_e_a;
irf_i  = oo_.irfs.i_e_a;
irf_pi = oo_.irfs.pi_e_a;
irf_rn = oo_.irfs.rn_e_a;
irf_r  = oo_.irfs.r_e_a;
irf_rk = oo_.irfs.rk_e_a;
irf_n  = oo_.irfs.n_e_a;
irf_q  = oo_.irfs.q_e_a;
irf_ep = oo_.irfs.ep_e_a;

end

if shock == 3;  % monetary policy shock
    
    shocks;
    var e_rn; stderr 0.0025;         // monetary policy shock
    end;

options_.nograph   = 1;
options_.nomoments = 1;
options_.noprint   = 1;

stoch_simul(irf=24) y c i n pi rn q ce rk r k ep;

irf_y  = oo_.irfs.y_e_rn;
irf_c  = oo_.irfs.c_e_rn;
irf_i  = oo_.irfs.i_e_rn;
irf_pi = oo_.irfs.pi_e_rn;
irf_rn = oo_.irfs.rn_e_rn;
irf_r  = oo_.irfs.r_e_rn;
irf_rk = oo_.irfs.rk_e_rn;
irf_n  = oo_.irfs.n_e_rn;
irf_q  = oo_.irfs.q_e_rn;
irf_ep = oo_.irfs.ep_e_rn;

end

end


end


if shock == 1; % government spending shock
    
    figure;
    
    nr = 3;
    nc = 2;
    
    subplot(nr,nc,1,'fontsize',12);
    
    plot(irf_y.*100,    'r','linewidth',3);
    hold on;
    plot(irf_y_noff.*100,    'b--','linewidth',3);
    grid;
    title('Output','fontsize',12);
    legend('with FA','without FA','Location','NorthEast');
    legend('boxoff');
    
    subplot(nr,nc,2,'fontsize',12);
    
    plot(irf_c.*100,    'r','linewidth',3);
    hold on
    plot(irf_c_noff.*100,    'b--','linewidth',3);
    grid;
    title('Consumption','fontsize',12);
    
    subplot(nr,nc,3,'fontsize',12);
    
    plot(irf_i.*100,    'r','linewidth',3);
    hold on
    plot(irf_i_noff.*100,    'b--','linewidth',3);
    grid;
    title('Investment','fontsize',12);
    
    subplot(nr,nc,4,'fontsize',12);
    
    plot(irf_q.*100,    'r','linewidth',3);
    hold on
    plot(irf_q_noff.*100,    'b--','linewidth',3);
    grid;
    title('Price of capital','fontsize',12);
    
    subplot(nr,nc,5,'fontsize',12);
    
    plot(irf_n.*100,    'r','linewidth',3);
    hold on
    plot(irf_n_noff.*100,    'b--','linewidth',3);
    grid;
    title('Net worth','fontsize',12);
    
    subplot(nr,nc,6,'fontsize',12);
    
    plot( irf_ep.*100,    'r','linewidth',3);
    hold on
    plot( irf_ep_noff.*100,    'b--','linewidth',3);
    ylim([-0.0025, 0.0005 ]);
    grid;
    title('External finance premium','fontsize',12);
    xlabel('quarters after shock','fontsize',12);
    
    
end


if shock == 2;  % technology shock
    
    figure;
    
    nr = 3;
    nc = 2;
    
    subplot(nr,nc,1,'fontsize',12);
    
    plot(irf_y.*100,    'r','linewidth',3);
    hold on;
    plot(irf_y_noff.*100,    'b--','linewidth',3);
    grid;
    title('Output','fontsize',12);
    legend('with FA','without FA','Location','NorthEast');
    legend('boxoff');
    
    subplot(nr,nc,2,'fontsize',12);
    
    plot(irf_c.*100,    'r','linewidth',3);
    hold on
    plot(irf_c_noff.*100,    'b--','linewidth',3);
    grid;
    title('Consumption','fontsize',12);
    
    subplot(nr,nc,3,'fontsize',12);
    
    plot(irf_i.*100,    'r','linewidth',3);
    hold on
    plot(irf_i_noff.*100,    'b--','linewidth',3);
    grid;
    title('Investment','fontsize',12);
    
    subplot(nr,nc,4,'fontsize',12);
    
    plot(irf_q.*100,    'r','linewidth',3);
    hold on
    plot(irf_q_noff.*100,    'b--','linewidth',3);
    grid;
    title('Price of capital','fontsize',12);
    
    subplot(nr,nc,5,'fontsize',12);
    
    plot(irf_n.*100,    'r','linewidth',3);
    hold on
    plot(irf_n_noff.*100,    'b--','linewidth',3);
    grid;
    title('Net worth','fontsize',12);
    
    subplot(nr,nc,6,'fontsize',12);
    
    plot( irf_ep.*100,    'r','linewidth',3);
    hold on
    plot( irf_ep_noff.*100,    'b--','linewidth',3);
    grid;
    title('External finance premium','fontsize',12);
    xlabel('quarters after shock','fontsize',12);
    
    
end


if shock == 3;  % monetary policy shock
    
    figure;
    
    nr = 3;
    nc = 2;
    
    subplot(nr,nc,1,'fontsize',12);
    
    plot(irf_r.*100,    'r','linewidth',3);
    hold on;
    plot(irf_r_noff.*100,    'b--','linewidth',3);
    grid;
    title('Real interest rate','fontsize',12);
    
    subplot(nr,nc,2,'fontsize',12);
    
    plot(irf_c.*100,    'r','linewidth',3);
    hold on
    plot(irf_c_noff.*100,    'b--','linewidth',3);
    grid;
    title('Consumption','fontsize',12);
    legend('with FA','without FA','Location','SouthEast');
    legend('boxoff');
    
    subplot(nr,nc,3,'fontsize',12);
    
    plot(irf_i.*100,    'r','linewidth',3);
    hold on
    plot(irf_i_noff.*100,    'b--','linewidth',3);
    grid;
    title('Investment','fontsize',12);
    
    subplot(nr,nc,4,'fontsize',12);
    
    plot(irf_q.*100,    'r','linewidth',3);
    hold on
    plot(irf_q_noff.*100,    'b--','linewidth',3);
    grid;
    title('Price of capital','fontsize',12);
    
    subplot(nr,nc,5,'fontsize',12);
    
    plot(irf_n.*100,    'r','linewidth',3);
    hold on
    plot(irf_n_noff.*100,    'b--','linewidth',3);
    grid;
    title('Net worth','fontsize',12);
    xlabel('quarters after shock','fontsize',12);
    
    subplot(nr,nc,6,'fontsize',12);
    
    plot( irf_ep.*100,    'r','linewidth',3);
    hold on
    plot( irf_ep_noff.*100,    'b--','linewidth',3);
    ylim([-0.05, 0.15 ]);
    grid;
    title('External finance premium','fontsize',12);
    xlabel('quarters after shock','fontsize',12);
    
    
end

 
%if shock == 1;
%    print fig_bgg_gov -depsc2;
%end

%if shock == 2;
%    print fig_bgg_tech -depsc2;
%end

%if shock == 3;
%    print fig_bgg_mon -depsc2;
%end

% close all;


 

 



