% Estimation of the Bernanke Gertler Gilchrist 
% "The Financial Accelerator in a Quantitative Business-Cycle Framework."
% bgg_estimation_L_V2.mod, version 2.1, January 14, 2014
% version 2: estimation of  standard deviation of the shocks only using simulated or actual data
% possibility of doing ML or Bayesian estimation
 
% Declare all the variables in the model, including the shocks processes
    var y c i g ce n rk r q k x a h pi rn ep;    

% Declare the exogenous variables, i.e. the innovations to the shocks g and a and the monetary shock
     varexo e_rn e_g e_a;                      
    
% declare the parameters of the model
    
    parameters beta alpha delt omeg GY CY IY CEY Y_N X rho_a sig_a rho_g sig_g psi K_N gam mu nu theta rho sig;
    
% Set values for the parameters
    
    CY      =	0.605770191     ; %  households' consumption - output ratio in steady state
    CEY  	=	0.01	        ; %  entrepreneurs' consumption - output ratio in steady state
    IY  	=	0.184229809     ; %  investment - output ratio in steady state
    GY  	=	0.2             ; %  government spending - output ratio in steady state
    K_N 	=	2.081759973     ; %  capital to net worth ratio in steady state
    Y_N 	=	0.282494996     ; %  output to net worth ratio in steady state
    X   	=	1.1             ; %  steady state firms' markup over marginal costs
    beta    =   0.99            ; %  households' discount factor
//  R       =   1/beta          ; %  Steady state nominal interest rate
    alpha   =   0.2             ; %  Capital share in the production function
//  eta     =   1/alpha         ;
    omeg    =   0.99            ; % Share of households' labor in production function. 1-omeg is the share of entrepreneurs'labor
    delt    =   0.025           ; % Depreciation rate of capital stock
    rho_a   =   0.99            ; % Persistene of technology process
    rho_g   =   0.95            ; % Persistene of government spending process
    psi     =   0.25            ; % Parameter in the adjustment cost function on capital
//  Rk      =   R + 0.02        ; % Return on capital = risk-free rate R + spread of 2 per cent per annum
    gam     =   1 - 0.0272      ; % Survival probability of entrepreneurs
    mu      =   0.12            ; % Monitoring cost
    theta   =   0.75            ; % Calvo probability of not being able to adjust prices
    rho     =   0.96            ; % Interest rate intertia: coefficient on rn(t-1)
    sig     =   0.20            ; % Response of monetary policy to inflation (t-1)
    nu      =   0.04            ; % sensitivity of the external finance premium to ratio of net worth to value of capital
    
//  kap     =  ((1-theta)/theta)*(1-theta*beta);
//  eps     =   (1-delt)/((1-delt) + ((alpha/X)*(Y_N/K_N)));
    

   % model specification; the model is already in linear form
   
    model(linear); 
   
%********L********L*********L*********L********L********L********L*********L*********L********
% parameters or steady state values that are (known) functions of other parameters (needed for estimation)
%********L********L*********L*********L********L********L********L*********L*********L********
 
# R=1/beta; 
# eta=1/alpha;
# Rk =   1/beta + 0.02        ; % Return on capital = risk-free rate R + spread of 2 per cent per annum
# kap     =  ((1-theta)/theta)*(1-theta*beta);
# eps     =   (1-delt)/((1-delt) + ((alpha/X)*(Y_N/K_N)));
    
% **************************************************************************
% equations numbers correspond to the BGG (1999) NBER Working Paper No. 6455
% **************************************************************************
    
    y             =  CY*c + IY*i + GY*g + CEY*ce                         ; % (4.14) RESOURCE CONSTRAINT
    
    c             =  -r + c(+1)                                          ; % (4.15) EULER EQUATION
    
    ce            =  n                                                   ; % (4.16) ENTREPRENEURS' CONSUMPTION
    
    rk(+1) - r    =  -nu*(n -(q + k))                                    ; % (4.17) RELATION BETWEEN EXTERNAL FINANCE PREMIUM AND NETWORTH
    
    rk            =  (1-eps)*(y - k(-1) - x) + eps*q - q(-1)             ; % (4.18) RETURN ON CAPITAL
    
    q             =  psi*(i - k(-1))                                     ; % (4.19)  q(+1) = psi*(i(+1)-k) PRICE OF NEWLY INSTALLED CAPITAL
    
    y             =  a + alpha*k(-1) + (1-alpha)*omeg*h                  ; % (4.20)  PRODUCTION FUNCTION
    
    y - h - x - c =  alpha*h                                             ; % (4.21)  LABOUR MARKET EQUILIBRIUM
    
    pi            =  kap*(-x) + beta*pi(+1)                              ; % (4.22)  NEW KEYNESIAN PHILLIPS CURVE
    
    k             =  delt*i(-1) + (1-delt)*k(-1)                         ; % (4.23) CAPITAL ACCUMULATION LAW
    
    n             =  gam*R*K_N*(rk - r) + r + n(-1)                      ; % (4.24) ACCUMULATION OF NETWORTH
    
    rn            =  rho*rn(-1) + sig*pi(-1) + e_rn                      ; % (4.25)  MONETARY POLICY RULE
    
    g             =  rho_g*g(-1) + e_g                                   ; % (4.26)  GOVERNMENT SPENDING PROCESS
    
    a             =  rho_a*a(-1) + e_a                                   ; % (4.27)  TECHNOLOGY PROCESS
    
    rn            =  r + pi(+1)                                          ; % EX-ANTE REAL INTEREST REATE
    
    ep            = -nu*(n -(q + k))                                     ; % definition external finance premium
    
end;        % ends the declaration of the model

steady;     % Computes the steady state of the model; in our case the steady state must be zero for all the variables

check;      % Checks for uniqueness of a stable equilibrium in neighborhood of steady state


%********L********L*********L*********%********L********L*********L*********L
%             Model estimation with financial accelerator                   L
%********L********L*********L*********L********L********L*********L*********L

estimation_procedure= input('Which estimation procedure? ML = 1; Bayesian-MCMC = 2 '); % Select the estimation procedure

estimation_dataset= input('Which dataset? Simulated = 1; True data = 2 '); % Select the Dataset

if estimation_dataset==1
// use simulated data
    shocks;
    var e_rn; stderr 0.0025;         // monetary policy shock
    var e_g; stderr 0.01;
    var e_a; stderr 0.01;  
    end;

stoch_simul(periods=400, nodisplay) y c i n pi rn q ce rk r k ep;
 
save bgg_estimation_data y c i n pi rn q ce rk r k ep

else           

// use actual data 

% the data is created in  the  file  detrend_ir2002.m
% which is called inside the program
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
detrend_IR2002
 
save bgg_estimation_data y pi rn 

end

%********L********************L
// varobs can only be declared once. Dynare does not support a conditional 
// selection of the dataset inside a .MOD file
%********L********************L
varobs y rn pi  ;

if estimation_procedure==1

    estimated_params;
// old structure: PARAM NAME, INITIAL VALUE, LOWER BOUND, UPPER BOUND
stderr e_rn,  0.0025,0.001,1.5;
stderr e_g,   0.01,0.00001,1.5;
stderr e_a,   0.01,0.00001,1.5;
    end;
// ML estimation
estimation(datafile=bgg_estimation_data,mode_compute=5) y rn pi ;

else
    estimated_params;
// PARAM NAME, PRIOR_SHAPE, PRIOR_P1, PRIOR_P2, PRIOR_P3, PRIOR_P4, JSCALE
// PRIOR_SHAPE: BETA_PDF, GAMMA_PDF, NORMAL_PDF, INV_GAMMA_PDF, UNIFORM_PDF
stderr e_rn,INV_GAMMA_PDF,0.0025,0.05;
stderr e_g,INV_GAMMA_PDF,0.01,0.05;
stderr e_a, INV_GAMMA_PDF,0.01,0.05;
    end;
// BAYESIAN ESTIMATION 
estimation(datafile=bgg_estimation_data,plot_priors=1,smoother, mode_compute=5,mode_check,mh_replic=10000,mh_nblocks=2,mh_jscale=1.2,mh_init_scale=0.5,bayesian_irf,moments_varendo) y rn pi ;


end




