function llfn = llfnd(thetvec);
%   Uses the Kalman filter to evaluate the negative log likelihood
%   function for a RBC model with indivisible labor.  
%   Parameters are transformed so that they satisty theoretical restrictions.  
%   The matrix V constrained to be diagonal.
 

  global yt ct ht

  bigt = length(yt);

  % read in initial conditions
  thetvec = real(thetvec);

  bettr = sqrt(0.99/(1-0.99));
  gamtr = thetvec(1);
  thettr = thetvec(2);
  etatr = thetvec(3);
  delttr = sqrt(0.025/(1-0.025));
  atr = thetvec(4);
  rhotr = thetvec(5);
  sigtr = thetvec(6);

  vyytr = thetvec(7);
  vcytr = 0;
  vcctr = thetvec(8);
  vhytr = 0;
  vhctr = 0;
  vhhtr = thetvec(9);

% untransform parameters

  beta = bettr^2/(1+bettr^2);
  gamma = abs(gamtr);
  theta = thettr^2/(1+thettr^2);
  eta = 1 + abs(etatr);
  delta = delttr^2/(1+delttr^2);
  a = abs(atr);
  rho = rhotr;
  sig = abs(sigtr);

% compute steady-state values

  kappa = eta/beta - 1 + delta;
  lambda = eta - 1 + delta;
  hss = ((1-theta)/gamma)/(1-theta*lambda/kappa);
  yss = (a^(1/(1-theta)))*((theta/kappa)^(theta/(1-theta)))*hss;
  kss = (theta/kappa)*yss;
  iss = (theta*lambda/kappa)*yss;
  css = yss - iss;

% compute K coefficients

  bigk11 = (eta-beta*(1-theta)*(1-delta))/(beta*eta*theta);

  bigk12 = (beta*eta*theta^2-eta+beta*(1-theta^2)*(1-delta)) ...
             /(beta*eta*theta^2);

  bigk22 = eta*theta/(eta-beta*(1-theta)*(1-delta));

% compute L coefficients

  bigl1 = (eta-beta*(1-delta))/(beta*eta*theta^2);

  bigl2 = rho*(eta-beta*(1-delta))/(eta-beta*(1-theta)*(1-delta));

% form S matrices

  bigs1 = (bigk22-bigk11)/bigk12;

  bigs2 = ((bigk22-bigk11)*bigl1-bigk12*bigl2)/(bigk12*(bigk11-rho));

  bigs3 = bigk22;

  bigs4 = bigk12*bigl2/(bigk22-bigk11) ...
            + (bigk22-rho)*((bigk22-bigk11)*bigl1-bigk12*bigl2) ...
            / ((bigk22-bigk11)*(bigk11-rho));

  bigs5 = [ 1 - ((1-theta)/theta)*bigs1 ; ...
            bigs1 + (kappa/(theta^2*lambda))*(theta-bigs1) ; ...
            1 - (1/theta)*bigs1 ];

  bigs6 = [ 1/theta - ((1-theta)/theta)*bigs2 ; ...
            bigs2 + (kappa/(theta^2*lambda))*(1-bigs2) ; ...
            1/theta - (1/theta)*bigs2 ];

% form matrices PI, W, and U of the blanchard and kahn solution

  bigpi = [ bigs3 bigs4 ; 0 rho ];

  bigw = [ 0 ; 1 ];

  bigu = [ bigs5 bigs6 ; bigs1 bigs2 ];

% adding measurement error to the control equations  

% form matrices AX, BX, CX, DX, V1X, and V2x

  bigax = bigpi;

  bigbx = bigw;

  bigcx = [ bigu(1,:) ; bigu(4,:) ; bigu(3,:) ];

  % this is the AR(1) coefficient matrix of the measurement errors
  bigdx = zeros(3,3);
       
  dxeig = eig(bigdx);

  dxviol = 0;

  if max(abs(dxeig)) > 1
    dxviol = 1;
  end

  bigv1x = sig^2;

  % this is the choleski factor variance covariance matrix of measurement errors
  bigv2x1 = [ vyytr 0 0 ; ...
              vcytr vcctr 0 ; ...
              vhytr vhctr vhhtr ];

  bigv2x = bigv2x1*bigv2x1';

% form matrices FX, GX, and QX
% these are the matrices of the transformed system
% x(t+1) = FX x(t) + GX u(t)
% c(t)= QX x(t)
% where now x(t) = [s(t), v(t)] and v(t) are measurement errors
%           u(t) = [espilon(t), e(t)] are the technology and the measurement shocks

  bigfx = [ bigax zeros(2,3) ; zeros(3,2) bigdx ];

  biggx = [ bigcx eye(3) ];

  bigqx = [ bigbx*bigv1x*bigbx' zeros(2,3) ; zeros(3,2) bigv2x ];

% put data in deviation from the labor augmenting trend

  trend = 1:bigt;

  ythat = log(yt) - log(eta)*trend' - log(yss);

  cthat = log(ct) - log(eta)*trend' - log(css);

  hthat = log(ht) - log(hss);

  dthat = [ ythat cthat hthat ];

% evaluate negative log likelihood

  xt = zeros(5,1);

  bigsig1 = inv(eye(25)-kron(bigfx,bigfx))*bigqx(:);

  bigsigt = reshape(bigsig1,5,5);

  llfn = (3*bigt/2)*log(2*pi);

  for t = 1:bigt

    ut = dthat(t,:)' - biggx*xt; % prediction error

    omegt = biggx*bigsigt*biggx'; % variance of the prediction error

    omeginvt = inv(omegt);

    llfn = llfn + (1/2)*(log(det(omegt))+ut'*omeginvt*ut);  % lof likelihood

    bigkt = bigfx*bigsigt*biggx'*omeginvt;   % kalman gain

    xt = bigfx*xt + bigkt*ut;  % prediction of next period state

    bigsigt = bigqx + bigfx*bigsigt*bigfx' ...
                - bigfx*bigsigt*biggx'*omeginvt*biggx*bigsigt*bigfx';  % riccati equation

  end

% penalize constraint violations

  if dxviol

    llfn = llfn + 1e8;

  end

  if abs(imag(llfn)) > 0

    llfn = real(llfn) + 1e8;

  end

  if abs(rhotr) > 1

    llfn = llfn + 1e8;

  end