% rbcml.m: Find ML estimates of an RBC model with indivisible labor, labor augmenting
%           technological progress (deterministic) and technology shocks.

%           It estimates the model adding three iid measurement errors to the
%           controls assuming diagonal covariance matrices (there is only one
%           shock in the model, estimation uses three variables)
%
% Note: When maximizing the log likelihood function, parameters are transformed to satisfy 
%       theoretical restrictions and the log likelihood with transformed parameters is in llfnd.m.
%       When calculating standard errors, the original parameters are used and  
%       the log likelihood of the untransformed parameters is llfndse.m.
%
%       Output: i)  estimates of untransformed parameter: paramstar.  
%               ii) standard errors of untransformed parameters: sevec.
%               iii) maximum of the likelihood: fstar.  
%               iv) with ML estimates, model is solved and responses to technology shocks calculated
%
%       This program is a modification of the estdse.m program written by Peter Ireland
%       you can find at this home page at Boston College
%
%%%%%%%%%%%%% Estimated Model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% max E sum(t) beta^t (log(c(t))-gamma N(t)
% subject to  c(t) + k(t+1)-(1-delta)k(t) = A(t) k(t)^(theta) (eta^t N(t))^{1-theta)
%             A(t) = (1-rho)A +rho A(t-1) +epsilon(t)    espilon(t) (0, sig^2)  
%            gamma>0 , 0<beta<1 , A>0,  -1< rho < 1, 0< theta< 1, 0 , delta < 1, eta > 1
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%  Few exercises to play around with the program %%%%%%%%%%%%%%% 
% exercise 1: Compute a likelihood ratio test for the hypothesis that agents do not discount the future (beta=1)
%           Hint: rerun the model with beta=1 and compare the max values of the likelihood to a chi^2(1)
% exercise 2: compute a likelihood ratio test that the estimates of two half subsamples are the same.
%           Hint: same as above, simply select the data appropriately in lines 47-49)
% exercise 3: compute variances and contemporaneous correlations of y, c, h, implied by ML estimates.
% exercise 4: change initial conditions. Does the routine converge to the same estimates?
%%  more difficult ones, requires some programming %%%
% exercise 5: use only two measurement errors
% exercise 6: add standard error bands to the impulse responses (by Monte
%             Carlo assuming a normal distribution centered at ML estimates)
% exercise 7: drop delta from the estimated parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
close all;

global yt ct ht scalinv

% a)  the data includes output (y), consumption (c) and hours (h) 
  load ych.dat;

  yt = ych(:,1);
  ct = ych(:,2);
  ht = ych(:,3);

% b) starting values for the optimization
  bettr = sqrt(0.99/(1-0.99));  % discount factor
  gamtr = 0.0045;               % constant in utility 
  thettr = sqrt(0.20/(1-0.20)); % share of capital in production fucntion 
  etatr = 0.0051;               % labor augmenting technological progress 
  delttr = sqrt(0.025/(1-0.025));  % depreciation rate
  atr = 6;                      % constant technology shock
  rhotr = 0.9914;               % ar(1) technology shock
  sigtr = 0.0073;               % standard deviation technology shock

  % measurement errors
  % variance matrix
  vyytr = 0.0101;
  vcytr = 0;
  vcctr = 0.0061;
  vhytr = 0;
  vhctr = 0;
  vhhtr = 0.0078;

  % c) parameters to be estimated
 thet0 = [ gamtr thettr etatr atr rhotr sigtr vyytr vcctr vhhtr ]';
               

 % d) unconstrained minimization of the log likelihood 

 % the next few lines works only with matlab versions less than 6.5
 %  setting up options for the maximization              
 % options(1) = 1;
 % options(14) = 10000;
 % thetstar = fminu('llfnd',thet0,options);

  
  % for versions of matlab above 6.5 use
  options=optimset('Display','iter','Diagnostics','on','MaxFunEval', 5000,'maxiter',100)
  thetstar = fminunc('llfnd',thet0,options);


% e) compute standard errors

  thetstar = real(thetstar);

  beta = bettr^2/(1+bettr^2);
  gamma = abs(thetstar(1));
  theta = thetstar(2)^2/(1+thetstar(2)^2);
  eta = 1 + abs(thetstar(3));
  delta = delttr^2/(1+delttr^2);
  a = abs(thetstar(4));
  rho = thetstar(5);
  sig = abs(thetstar(6));

  cholv = [ thetstar(7) 0 0 ; ...
            0 thetstar(8) 0 ; ...
            0 0 thetstar(9) ];

  bigv = cholv*cholv';

  vyy = sqrt(bigv(1,1));
  vcc = sqrt(bigv(2,2));
  vhh = sqrt(bigv(3,3));
  vyc = bigv(1,2);
  vyh = bigv(1,3);
  vch = bigv(2,3);

 
  paramstar = [ gamma theta eta a rho sig vyy vcc vhh ]';

  scalvec = ones(9,1);
  scalinv = inv(diag(scalvec));
  tstars = diag(scalvec)*paramstar;

  % evaluation of the likelihood at the estimated parameters
   fstar = llfndse(tstars);

  eee = 1e-6;
  epsmat = eee*eye(9);
  hessvec = zeros(9,1);

  for i = 1:9;
    hessvec(i) = llfndse(tstars+epsmat(:,i));
  end

  hessmat = zeros(9,9);
  for i = 1:9;
      for j = 1:9;
          hessmat(i,j) = (llfndse(tstars+epsmat(:,i)+epsmat(:,j)) ...
                        -hessvec(i)-hessvec(j)+fstar)/eee^2;
      end
  end

  bighx = scalinv*inv(hessmat)*scalinv';

  sevec = sqrt(diag(bighx));
  

% Results
% *******************************************************************************
disp(' ')
disp(' ')
disp('************************************')
disp('CALIBRATED PARAMETERS')
disp('************************************')
disp([' beta    = ' num2str(beta) ''])
disp([' delta    = ' num2str(delta) ''])
disp('************************************')
disp('ESTIMATED PARAMETERS')
disp('************************************')
disp([' gamma    = ' num2str(paramstar(1)) ''])
disp(['          (' num2str(sevec(1)) ')'])
disp([' theta    = ' num2str(paramstar(2)) ''])
disp(['          (' num2str(sevec(2)) ')'])
disp(['  eta      = ' num2str(paramstar(3)) ''])
disp(['          (' num2str(sevec(3)) ')'])
disp([' a        = ' num2str(paramstar(4)) ''])
disp(['          (' num2str(sevec(4)) ')'])
disp([' rho      = ' num2str(paramstar(5)) ''])
disp(['          (' num2str(sevec(5)) ')'])
disp([' sigma    = ' num2str(paramstar(6)) ''])
disp(['          (' num2str(sevec(6)) ')'])
disp('************************************')
disp('MEASUREMENT ERRORS')
disp('************************************')
disp([' sigmae1    = ' num2str(paramstar(7)) ''])
disp(['          (' num2str(sevec(7)) ')'])
disp([' sigmae2    = ' num2str(paramstar(8)) ''])
disp(['          (' num2str(sevec(8)) ')'])
disp([' sigmae3      = ' num2str(paramstar(9)) ''])
disp(['          (' num2str(sevec(9)) ')'])
% *******************************************************************************

  
  % f) solve now the model with ML estimated parameters
  %    use Blanchard and Kahn method (see Ireland notes: notes.pdf)
  
% f.1) compute steady-state values

  kappa = eta/beta - 1 + delta;
  lambda = eta - 1 + delta;
  hss = ((1-theta)/gamma)/(1-theta*lambda/kappa);
  yss = (a^(1/(1-theta)))*((theta/kappa)^(theta/(1-theta)))*hss;
  kss = (theta/kappa)*yss;
  iss = (theta*lambda/kappa)*yss;
  css = yss - iss;

% compute K coefficients

  bigk11 = (eta-beta*(1-theta)*(1-delta))/(beta*eta*theta);

  bigk12 = (beta*eta*theta^2-eta+beta*(1-theta^2)*(1-delta)) ...
             /(beta*eta*theta^2);

  bigk22 = eta*theta/(eta-beta*(1-theta)*(1-delta));

% compute L coefficients

  bigl1 = (eta-beta*(1-delta))/(beta*eta*theta^2);

  bigl2 = rho*(eta-beta*(1-delta))/(eta-beta*(1-theta)*(1-delta));

% form S matrices

  bigs1 = (bigk22-bigk11)/bigk12;

  bigs2 = ((bigk22-bigk11)*bigl1-bigk12*bigl2)/(bigk12*(bigk11-rho));

  bigs3 = bigk22;

  bigs4 = bigk12*bigl2/(bigk22-bigk11) ...
            + (bigk22-rho)*((bigk22-bigk11)*bigl1-bigk12*bigl2) ...
            / ((bigk22-bigk11)*(bigk11-rho));

  bigs5 = [ 1 - ((1-theta)/theta)*bigs1 ; ...
            bigs1 + (kappa/(theta^2*lambda))*(theta-bigs1) ; ...
            1 - (1/theta)*bigs1 ];

  bigs6 = [ 1/theta - ((1-theta)/theta)*bigs2 ; ...
            bigs2 + (kappa/(theta^2*lambda))*(1-bigs2) ; ...
            1/theta - (1/theta)*bigs2 ];

  % f.2) form matrices PI, W, and U
  % format is f(t+1)= PI f(t) + W e(t)
  %           s(t) = U f(t)
  
  bigpi = [ bigs3 bigs4 ; 0 rho ];

  bigw = [ 0 ; 1 ];

  bigu = [ bigs5 bigs6 ; bigs1 bigs2 ];
  
  
  % f.3) responses to technology shock

  horiz=40;
  st = zeros(horiz,2);
  ft = zeros(horiz,4);

  et = sig;

  st(2,:) = (bigw*et)';
  ft(2,:) = (bigu*st(2,:)')';

  for t = 3:horiz;
    st(t,:) = (bigpi*st(t-1,:)')';
    ft(t,:) = (bigu*st(t,:)')';
  end

% create output vectors from states (st) and controls (ft) matrices

  st = 100*st;
  ft = 100*ft;

  kt = st(:,1);
  at = st(:,2);

  yt = ft(:,1);
  it = ft(:,2);
  ht = ft(:,3);
  ct = ft(:,4);
  
% plot responses to a 1% shock

jj=[0:horiz-1];
subplot(3,2,1)
plot(jj,yt(1:horiz,:),'-') % 
xlabel('output')

subplot(3,2,2)
plot(jj,it(1:horiz,:),'-') % 
xlabel('investment')
 

subplot(3,2,3)
plot(jj,ct(1:horiz,:),'-') % 
xlabel('Consumption')

subplot(3,2,4)
plot(jj,ht(1:horiz,:),'-') % 
xlabel('Hours')

 
subplot(3,2,5)
plot(jj,kt(1:horiz,:),'-') % 
xlabel('Capital')

subplot(3,2,6)
plot(jj,at(1:horiz,:),'-') % 
xlabel('techology')
 
 
